@@ -289,7 +289,7 @@ plotCore <- function(iplot, scrfine, SfdList, dataList, Qvec,
       # print("itemStr")
       if (!is.null(titlestr)) {
         # a scale title is supplied
-        # print("titlestr")
+        print("titlestr")
         if (scopei != 0) {
           # print("scope")
           # a vector of scope value is supplied
@@ -302,7 +302,7 @@ plotCore <- function(iplot, scrfine, SfdList, dataList, Qvec,
                           round(scopei,1),sep="")
         }
       } else {
-        # print("no itemstr")
+        # print("no itemstr line 305")
         if (!(scopei == 0)) {
           # print("scope")
           # a vector of scope value is supplied
@@ -317,7 +317,7 @@ plotCore <- function(iplot, scrfine, SfdList, dataList, Qvec,
     } else 
       if (!is.null(titlestr)) {
         # a scale title is supplied
-        # print("titlestr")
+        # print("titlestr line 320")
         if (!(scopei == 0)) {
           # print("scope")
           # a vector of scope value is supplied
@@ -330,8 +330,7 @@ plotCore <- function(iplot, scrfine, SfdList, dataList, Qvec,
         }
       } else {
         # print("no titlestr")
-        # print(scopei)
-        if (!(scopei == 0)) {
+        if (!(is.na(scopei))) {
           # print("scope")
           # a vector of scope value is supplied
           ttllab <- paste("item",iplot,': ',"scope ",
